/**
 * M1k2 logfile
 * @copyright D.O.Tec, http://www.directout.eu
 * @version 1 (18.03.2012) Florian Faber, florian.faber@directout.eu
 **/

var log_types = ["status", "config", "warn"];

function gain_to_db(gainval) {
  return 20*(Math.log(gainval/2048.0)/2.30258509299);
}



/**
 * 
 * EventSource callbacks
 *
 **/

function ess_key_valid(val) {
	add_log("Device key valid: "+((val)?"yes":"no"), 'status');
}

function ess_board(id) {
	add_log("Mainboard ID "+id, 'status');
}

function ess_serial(serial) {
	add_log("Mainboard serial "+serial, 'status');
}

function ess_firmware(major, minor, build) {
	add_log("FPGA firmware "+major+"."+minor+", build "+build, 'status');
}

var io_ports = ["N/C", "BNC", "SC Multimode", "SC Singlemode", "ST Multimode", "ST Singlemode", "SFP"];

function ess_io(id, p1, p2, p3, p4, p5, p6, p7, p8) {
	add_log("IO Board #"+(id+1)+": "+io_ports[p1]+"/"+io_ports[p2]+"/"+io_ports[p3]+"/"+io_ports[p4]+"/"+io_ports[p5]+"/"+io_ports[p6]+"/"+io_ports[p7]+"/"+io_ports[p8], 'status');
}

function ess_io_present(id, p) {
	add_log("IO Board #"+(id+1)+" "+((p==0)?"not":"")+" present", 'status');
}

function ess_io_power(id, p) {
	add_log("IO Slot #"+(id+1)+" "+((p==0)?"not":"")+" powered", 'status');
}

function ess_psu(id, s) {
	add_log("PSU #"+(id+1)+": "+((s==0)?"OFF":(s==1)?"ON":"FAILED"), (s==2)?'warn':'status');
}

var last_temp = 0;

function ess_temp(temp) {
  temp = Math.floor(temp/1000);
  if (temp!=last_temp) {
    add_log("Mainboard temperature "+temp+" degrees Celsius", 'status');
    last_temp = temp;
  }
}

function ess_fan(speed) {
	add_log("Fan "+speed+"%", 'status');
}

function ess_freq(port, freq) {
	add_log("Frequency on Port "+(port+1)+" now "+freq+"Hz", 'status');
}

function ess_port_mode(port, mode) {
	add_log("Port "+(port+1)+" input mode "+mode+"ch", 'status');
}

function ess_port_frame(port, frame) {
	add_log("Port "+(port+1)+" input frame "+frame+"k", 'status');
}

sync_states = ["No signal", "Lock", "??", "Sync"];

function ess_sync(port, sync, t) {
	add_log("Sync on port "+(port+1)+": "+sync_states[sync], 'status');
}

function ess_video_mode(mode) {
	add_log("Video mode changed, is now "+mode, 'status');
}

function ess_hwaddr(str) {
  add_log("MAC address "+str, 'config');
}

function ess_polysync(p) {
	add_log("PolySync "+(p?"active":"inactive"), 'status');
}

function ess_usage(fs, usage) {
	add_log("File system usage on "+((fs==0)?"root":"user")+" partition: "+usage+"%", 'status');
}


//
//
//


function esc_master_enable_clock(enable) {
	add_log("Master clock source "+(enable?"enabled":"disabled"), 'config');
}

function esc_master_enable_fs(enable) {
	add_log("Master clock FS "+(enable?"enabled":"disabled"), 'config');
}

function esc_xp(m, ch_in, ch_out) {
	if (m==0) {
		if (ch_in>=0) {
			add_log("Audio XP "+(ch_in+1)+"->"+(ch_out+1), 'config');
		} else {
			add_log("Audio XP "+(ch_out+1)+" deactivated", 'config');
		}
	} else {
		if (ch_in>=0) {
			add_log("Offline XP "+(ch_in+1)+"->"+(ch_out+1), 'config');
		} else {
			add_log("Offline XP "+(ch_out+1)+" deactivated", 'config');
		}
	}
}

function esc_al(ch, val) {
	add_log("Channel lock "+(ch+1)+": "+((val==0)?"un":"")+"locked", 'config');
}

function esc_gl(ch, val) {
	add_log("Gain lock "+(ch+1)+": "+((val==0)?"un":"")+"locked", 'config');
}

function esc_gain(ch, gain) {
	add_log("Channel gain "+(ch+1)+": "+Math.round(gain_to_db(gain)*100)/100, 'config');
}

function esc_midixp(p_in, p_out) {
	if (p_in<31) {
		add_log("MIDI XP "+(p_in+1)+"->"+(p_out+1), 'config');
	} else {
		add_log("MIDI XP "+(p_out+1)+" deactivated", 'config');
	}
}


function esc_serxp(p_in, p_out) {
	if (p_in<31) {
		add_log("Serial XP "+(p_in+1)+"->"+(p_out+1), 'config');
	} else {
		add_log("Serial XP "+(p_out+1)+" deactivated", 'config');
	}
}

baud_rates = ["9600", "19200", "38400", "115200"];

function esc_serbaud(p, baud) {
	add_log("Baud rate on port "+(p+1)+" changed to "+baud_rates[baud]+" Bit/s", 'config');
}


wck_sources = ["Port 1", "Port 2", "Port 3", "Port 4",
							 "Port 5", "Port 6", "Port 7", "Port 8",
							 "Port 9", "Port 10", "Port 11", "Port 12",
							 "Port 13", "Port 14", "Port 15", "Port 16",
							 "WCK Input", "Video 44.1kHz", "Video 48kHz", "Internal 44.1kHz",
							 "Internal 48kHz"];

port_names = ["Port 1", "Port 2", "Port 3", "Port 4",
							"Port 5", "Port 6", "Port 7", "Port 8",
							"Port 9", "Port 10", "Port 11", "Port 12",
							"Port 13", "Port 14", "Port 15", "Port 16",
							"Word clock", "Master clock"];

function esc_wck_source(source) {
	add_log("Wordclock output set to "+wck_sources[source], 'config');
}

function esc_set_polysync(p) {
	add_log("Polysync "+((p==1)?"on":"off"), 'config');
}

function esc_poly_source(port, source) {
	add_log("Clock reference for port "+(port+1)+" set to "+wck_sources[source], 'config');
}

function esc_wck_mul(port, mul) {
	add_log("S/MUX mode for port "+(port+1)+" is now "+((mul==2)?4:(mul+1))+"FS", 'config');
}

function esc_port_mode(port, mode) {
	add_log("Output mode for port "+(port+1)+" set to "+mode+"ch", 'config');
}

function esc_port_frame(port, frame) {
	add_log("Output frame for port "+(port+1)+" set to "+frame+"k", 'config');
}

function esc_port_input_redundancy(port, redundancy) {
	add_log("Redundancy input port for port "+(port+1)+" is now "+((redundancy==port)?"off":("port "+(redundancy+1))), 'config');
}

function esc_port_output_redundancy(port, redundancy) {
	add_log("Redundancy output port for port "+(port+1)+" is now "+((redundancy==port)?"off":("port "+(redundancy+1))), 'config');
}

function esc_port_follow(port, redundancy) {
	add_log("Port follow for "+port_names[port]+" is now "+((redundancy==port)?"off":("port "+(redundancy+1))), 'config');
}

function esc_gpo(gpo, level) {
	add_log("GPO "+(gpo+1)+" is now "+((level==1)?"on":"off"), 'config');
}

var gpo_automations = ["manual activation", "port redundancy triggered", "full temperature reached", "critical temperature reached", "sync loss on any port", "signal loss on wordclock input", "signal loss on video input", "psu 1 fail", "psu 2 fail", "any PSU fail", "MMC play", "MMC record"];

function esc_gpo_automation(gpo, action) {
	add_log("GPO "+(gpo+1)+" automation set to '"+gpo_automations[action]+"'", 'config');
}

function esc_gpo_polarity(gpo, pol) {
	add_log("GPO "+(gpo+1)+" polarity "+((pol==1)?"inverted":"normal"), 'config');
}

function esc_term(level) {
	add_log("Term is now "+level, 'config');
}

function esc_blink_unsafed(val) {
	add_log("State LED blinking when unsafed data present "+((val==1)?"activated":"disabled"), 'config');
}

function esc_state_idle(val) {
	add_log("State LED heartbeat "+((val==1)?"activated":"disabled"), 'config');
}

var msg_levels = ["EMERG", "ALERT", "CRIT", "ERR", "WARNING", "NOTICE", "INFO", "DEBUG"];

function esc_msg_level(val) {
	add_log("Log level set to "+msg_levels[val], 'config');
}

function esc_fan(warm, full, critical) {
	add_log("Fan configuration set to [warm="+warm+", full="+full+", critical="+critical+"]", 'config');
}

function esc_portgain(port, gain) {
	add_log("Port gain on port "+(port+1)+": "+Math.round(gain_to_db(gain)*100)/100, 'config');
}

function esc_portgain_mode(port, mode) {
	add_log("Port gain mode for port "+(port+1)+" is now "+mode, 'config');
}

function set_profile(id) {
	add_log("Profile #"+id+" activated", 'config');
}

function set_network(id) {
	add_log("Network configuration #"+id+" activated", 'config');
}

function set_name(str) {
	add_log("Device name set to '"+str+"'", 'config');
	document.title = "LiveLog "+str;
}

function set_contact(str) {
	add_log("Device contact set to '"+str+"'", 'config');
}

function syslog(timestamp, facility, severity, appname, msg) {
  switch (severity) {
  case 0: str = "EMERGENCY"; break;
  case 1: str = "ALERT"; break;
  case 2: str = "CRITICAL"; break;
  case 3: str = "ERROR"; break;
  }
  add_log(str+": ["+appname+"] "+msg, 'warn');
}


function log_disconnected() {
  o = document.getElementsByTagName("body")[0];
  fade_set_opacity(o, 0.5);

}

function log_connected() {
  o = document.getElementsByTagName("body")[0];
  if (o && fade_set_opacity)
    fade_set_opacity(o, 1);
}


function livelog_init() {
  update_eventcounter();
  ws_init(log_connected, log_disconnected);
}

