/**
 * @copyright Florian Faber, florian.faber@directout.eu
 * @version 2 (05.05.2013) 
 *          - added permissions
 **/

var bg_mixed = '606060';
var bg_act = '009000';
var bg_forbidden = '900000';
var bg_warn = '909000';
var bg_nil = '';
var labels;
var portstate = Array(16);
var port_locked = Array(16);
var gainstate = Array(16);
var locks;
var input_active = false;


function remove_all_mixed(port) {
    var c = document.getElementById('matrix_xps');
    
    for (i=0; i<16; i++) {
	var id = 'mixed_'+port+'_'+i;
	var o = document.getElementById(id);
	if (o)
	    c.removeChild(o);
    }
}


function draw_portstate(port) {
    switch (portstate[port]) {
    case 0: // N/C
	mini_xp(port, -1);
	break;
	
    case 1: // 1:1
	mini_xp(port, Math.floor(parent.frames[0].config_xpa[port*64]/64));
	break;

    default: // mixed
	mini_xp(port, -2); // hide regular XP
	
	var srcs = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0];
	for (var i=0; i<64; i++) {
	    var src = Math.floor(parent.frames[0].config_xpa[port*64+i]/64);
	    if (src>=0)
		srcs[src]++;
	}

	var c = document.getElementById('matrix_xps');

	for (i=0; i<16; i++) {
	    var id = 'mixed_'+port+'_'+i;
	    if (srcs[i]==0) {
		var o = document.getElementById(id);
		if (o)
		    c.removeChild(o);
	    } else {
		var o = document.getElementById(id);
		if (!o) {
		    o = document.createElement('div');
		    o.id = id;
		    o.className = 'mixedxp';
		    o.style.top = ((i+1)*23+1)+'px';
		    o.style.left = (port*23+1)+'px';
		    c.appendChild(o);
		}
	    }
	}
    }
}


function set_portstate(port, newstate) {
    if (newstate != portstate[port]) {
	if (portstate[port]!=1)
	    remove_all_mixed(port);
	portstate[port] = newstate;
	draw_portstate(port);
    } else if (newstate==2) {
	draw_portstate(port);
    }
}


function check_portstates() {
    for (var p=0; p<16; p++) {
	if (portstate[p] == -3) {
	    check_portstate(p);
	    portstate[p] = 0;
	}
    }
    timeout_xp = false;
    //timeout_xp = window.setTimeout("check_portstates()", 50);	
    
}


function ess_snapshot_loaded(snap) {
    for (var i=0; i<16; i++) {
	portstate[i] = -1;
	check_portstate(i);
    }
}


function check_portstate(port) {
    var start = parent.frames[0].config_xpa[port*64];
    var test_max = 64;
    if (parent.frames[0].get_effective_mul(port)==1)
	test_max = 32;
    else if (parent.frames[0].get_effective_mul(port)==2)
	test_max = 16;

    if (start==-1) {
	for (var i=1; i<test_max; i++) {
	    if (parent.frames[0].config_xpa[port*64+i]!=-1) {
		set_portstate(port, 2); // mixed
		return;
	    }
	}
	set_portstate(port, 0); // N/C

    } else {
	if (start!=(Math.floor(start/64)*64)) {
	    set_portstate(port, 2); // mixed (misaligned start)
	    return;
	}
	for (var i=1; i<test_max; i++) {
	    if (parent.frames[0].config_xpa[port*64+i]!=start+i) {
		set_portstate(port, 2); // mixed
		return; 
	    }
	}
	set_portstate(port, 1); // 1:1
    }

    timeout_xp = false;
}


function draw_lockstate(port) {
    var img = document.getElementById('l_'+port);
    switch(port_locked[port]) {
    case 0: // no locks
	img.style.opacity = 0;
	break;
    case 1: // all ch locked
	img.style.opacity = 1;
	break;
    default: // some ch locked
	img.style.opacity = 0.5;
    }
}


function update_lock_state(port) {
    var i, newstate, statecnt=[0,0];
    
    for (i=0; i<64; i++)
	statecnt[parent.frames[0].config_lock[port*64+i]]++;

    if (statecnt[0]==64)
	newstate = 0;
    else if (statecnt[1]==64)
	newstate = 1;
    else
	newstate = 2;

    if (newstate != port_locked[port]) {
	port_locked[port] = newstate;
	draw_lockstate(port);
    }

    timeout_lock = false;
}


function draw_gain(port, db, mode) {
    var g = document.getElementById('g_'+port);
    if (!g)
	return;

    if (db<0) {
	g.style.backgroundColor = 'green';
	g.style.color = 'white';
    } else if (db>0) {
	g.style.backgroundColor = 'yellow';
	g.style.color = 'red';
    } else { 
	g.style.backgroundColor = '#999999';
	g.style.color = 'black';
    }
    
    if (db<-67) {
	g.innerHTML = '&#8734;';
    } else if (Math.abs(db)>10) {
	g.innerHTML = Math.round(Math.abs(db));
    } else {
	var dbval = Math.round(Math.abs(db*10));
	g.innerHTML = dbval/10;
    }

    if (parent.frames[0].permissions.portgain[port]==0) {
	//g.style.opacity = 0.5;
	g.className = "disabled";
    } else {
	g.className = "";
    }
    //g.style.opacity = 1;
}


function update_gain_state(port) {
    var db = fader_gain_to_db(parent.frames[0].config_portgains[port]);
    var fader = document.getElementById('fader_'+port);
    draw_gain(port, db, false);

    if (fader && !fader_drag_active(fader)) {
	fader_set_knob(fader, db);
    }
}


function update_portgain_mode(port) {
    var img = document.getElementById('pgm_'+port);
    if (img) 
	img.src = "/img/portgainmode_"+parent.frames[0].config_portgain_modes[port]+".png";
}


function show_portgainmode(show) {
    document.getElementById('gainmode').style.display = show?"block":"none";
    document.getElementById('gainlabel').style.display = show?"none":"block";

    for (var i=0; i<16; i++) {
	document.getElementById('pgm_'+i).style.display = show?"block":"none";
	document.getElementById('g_'+i).style.display = show?"none":"block";
    }
}


//
// events
//


function lock_mousedown(e) {
    if (e.which!=1)
	return;

    var srcElement;

    if (e.srcElement) {
	srcElement = e.srcElement;
    } else if (e.originalTarget) {
	srcElement = e.originalTarget;
    }

    if (e.shiftKey) {
	if (port_locked[srcElement.port]==1)
	    send_cmd("lock_port?"+srcElement.port+",0");
	else
	    send_cmd("lock_port?"+srcElement.port+",1");
    } else {
	alert("Please hold down the shift key while klicking on the port lock row to enable the function.");
    }
}


function gainmode_mousedown(e) {
    if (e.which!=1)
	return;

    var srcElement;

    if (e.srcElement) {
	srcElement = e.srcElement;
    } else if (e.originalTarget) {
	srcElement = e.originalTarget;
    }

    send_cmd("portgain_mode?"+srcElement.port+","+(1-parent.frames[0].config_portgain_modes[srcElement.port]));
}


function gain_mousedown(e) {
    var srcElement;

    if (e.srcElement) {
	srcElement = e.srcElement;
    } else if (e.originalTarget) {
	srcElement = e.originalTarget;
    }

    if (parent.frames[0].permissions.portgain[srcElement.port]==0)
	return;

    if (e.which==1) {
	if (e.ctrlKey) {
	    gain_changed(srcElement.port, 0);
	    fader_set_gain(document.getElementById('fader_'+srcElement.port), 53);
	} else {
	    var cs = document.getElementById('fader_'+srcElement.port);
	    if (cs && (cs.style.display=='block')) {
		cs.style.display = 'none';
	    } else {
		fader_mono_popup(document.getElementById('contents'), 'fader_'+srcElement.port, srcElement.port, fader_gain_to_db(parent.frames[0].config_portgains[srcElement.port]), 165+23*srcElement.port, 625, (srcElement.port+1), gain_changed);
	    }
	}
	
    } else if (e.which==2) {
	gain_changed(srcElement.port, 0);
	fader_set_gain(document.getElementById('fader_'+srcElement.port), 53);
    }
}


function cb_keydown(e) {
    if (input_active)
	return;
    
    if (e.keyCode==71) { // 'g'
	var allvisible = true;
	for (var i=0; i<16; i++) {
	    if (parent.frames[0].permissions.portgain[i]==1) {
		var o = document.getElementById('fader_'+i);
		if (!o || (o.style.display=='none')) {
		    allvisible = false;
		    break;
		}
	    }
	}
	
	if (allvisible) {
	    for (var i=0; i<16; i++) {
		var f = document.getElementById('fader_'+i);
		if (f)
		    f.style.display = 'none';
	    }
	} else {
	    for (var i=0; i<16; i++) {
		if (parent.frames[0].permissions.portgain[i]==1) {
		    fader_mono_popup(document.getElementById('contents'), 'fader_'+i, i, fader_gain_to_db(parent.frames[0].config_portgains[i]), 165+23*i, 625, (i+1), gain_changed);
		}
	    }
	} 
	
    } else if (e.keyCode==68) { // 'd'
	push_cycle_button(document.getElementById("show_labels"));    
	
    } else if (e.keyCode==16) { // shift
	show_portgainmode(true);

    } else if (e.which==82) { // 'r'
	restore();

    } else if (e.which==83) { // 's'
	snapshot();

    } else {
	//alert(e.keyCode);
	return;
    }

    e.preventDefault();
}


function cb_keyup(e) {
    if (e.keyCode==16)
	show_portgainmode(false);
}


//
// callback for fader.js
//


function gain_changed(port, db) {
    //draw_gain(port, db, true);
    send_cmd("portgain?"+port+","+fader_db_to_gain(db));
}


//
// callback functions for status.js
//


var timeout_xp=false;


function esc_xp(m, ch_in, ch_out) {
    //check_portstate(Math.floor(ch_out/64));

    var port = Math.floor(ch_out/64);
    portstate[port] = -3;

    //if (timeout_xp) 
    //		window.clearTimeout(timeout_xp);

    if (!timeout_xp)
	timeout_xp = window.setTimeout("check_portstates()", 100);	
}


var timeout_gain=false; 

function esc_al(ch, val) {
    update_lock_state(Math.floor(ch/64));
    /*
      var port = Math.floor(ch/64);
      if (timeout_lock) 
      window.clearTimeout(timeout_lock);

      timeout_lock = window.setTimeout("update_lock_state("+port+")", 50);
    */
}


function esc_gl(ch, val) {

}

function esc_portgain_mode(port, mode) {
    update_portgain_mode(port);
}

function esc_portgain(port, gain) {
    update_gain_state(port);
}


function esl(type, id, label, fg, bg) {
    if (!labels)
	return;

    if (type==134) { // input port
	mini_update_in(id);

    } else if (type==135) { // output port
	mini_update_out(id);

    }
}


function esc_iomask_ports(mask) {
    var size = 23;
    var c = document.getElementById('matrix_bg');
    if (c && c.getContext) {
	var ctx = c.getContext('2d');
	ctx.clearRect(0, 0, 16*size, 16*size);
	for (var y=0; y<16; y++) {
	    for (var x=0; x<16; x++) {
		if (mask[x][y]==1) {
		    ctx.fillStyle = "rgb(128, 0, 0)";
		    ctx.fillRect(x*size+1, (y+1)*size+1, size-1, size-1);
		} else if (mask[x][y]==2) {
		    ctx.fillStyle = "rgb(128, 128, 0)";
		    ctx.fillRect(x*size+1, (y+1)*size+1, size-1, size-1);
		}										
	    }
	}
    }
}


//
// callback functions for minimatrix.js
//


function mini_cb_input(active) {
    input_active = active;
}


function mini_cb_color(dst, src) {
    if (mini_cb_enabled('src', src) && mini_cb_enabled('dst', dst)) {
	if (src>=0) {
	    if (parent.frames[0].iomask_ports[src][dst]==0) {
		return bg_act;
	    } else if (parent.frames[0].iomask_ports[src][dst]==2) {
		return bg_warn;
	    }
	} else {
	    return bg_act;
	}
    }
    
    return bg_forbidden;
}


function mini_cb_cursor(dst, src) {
    if (mini_cb_enabled('src', src) && mini_cb_enabled('dst', dst))
	return "pointer";
    
    return "not-allowed";
}

function mini_cb_click(dst, src, e) {
    if ((portstate[dst]==1) && (parent.frames[0].config_xpa[dst*64]==src*64))
	src = -1;
    
    if (mini_cb_enabled('src', src) && mini_cb_enabled('dst', dst)) {				
	if (src>=0) {
	    if (parent.frames[0].iomask_ports[src][dst]==1) {
		return;
	    }
	}
	if ((portstate[dst]==2) &&  (e.shiftKey==0)) {
	    alert("Destination has mixed sources. Please hold down the shift key while clicking!");
	    return;
	}
	
	send_cmd("xp_port?"+src+","+dst+",0");
    }
}


function mini_cb_label_editable(type, id) {
    return (labels==1);
}


function mini_cb_label_changed(type, id, newlabel, fg, bg) {
    post_cmd("label?"+((type=='src')?134:135)+","+id+","+fg+","+bg, newlabel);
}


var	names_hw = ["Port 1", "Port 2", "Port 3", "Port 4",
		    "Port 5", "Port 6", "Port 7", "Port 8",
		    "Port 9", "Port 10", "Port 11", "Port 12",
		    "Port 13", "Port 14", "Port 15", "Port 16",
		   ];


function mini_cb_label(type, id) {
    if (id==-1) {
	return "N/C";
    } else {
	if (labels==1) {
	    if (type=='src')
		return parent.frames[0].labels_input_ports[id];
	    else
		return parent.frames[0].labels_output_ports[id];
	} else {
	    return names_hw[id];
	}
    }
}

function mini_cb_enabled(type, id) {
    if (type=='src') {
	if (id==-1)
	    return true;
	return (parent.frames[0].permissions.ports_in[id]==1);
    } else {
	return (parent.frames[0].permissions.ports_out[id]==1);
    }
}


function mini_cb_bg(type, id) {
    if (labels==1) {
	if (type=='src') 
	    return parent.frames[0].labels_input_ports_bg[id];
	else
	    return parent.frames[0].labels_output_ports_bg[id];
    } else {
	return '';
    }
}


function mini_cb_fg(type, id) {
    if (labels==1) {
	if (type=='src') 
	    return parent.frames[0].labels_input_ports_fg[id];
	else
	    return parent.frames[0].labels_output_ports_fg[id];
    } else {
	return '';
    }
}


//
// internal
//



function show_labels(v) {
    if (typeof(v)=='string') {
	v = parseInt(v, 10);
	var bla=typeof(v);
	if ((typeof(v)!='number') || isNaN(v))
	    v = 0;
	set_cycle_button(document.getElementById("show_labels"), v);
    } else if (labels != v) {
	post_call('/wwwdata?show_labels', v, false);
    }
    
    labels = v;
    
    for (var i=0; i<16; i++) {
	mini_update_in(i);
	mini_update_out(i);
    }
}


function set_11() {
    for (var i=0; i<16; i++)
	send_cmd("xp_port?"+i+","+i+",0");
}

function set_nc() {
    for (var i=0; i<16; i++) 
	send_cmd("xp_port?-1,"+i+",0");
}


function check_gains() {
    if (typeof(parent.frames[0].config_portgains[15])=='number') {
	for (var i=0; i<16; i++) {
	    update_gain_state(i);
	    update_portgain_mode(i);
	}
	
    } else {
	window.setTimeout(check_gains, 50);
    }
}

function permissions_updated() {
    mini_update_all();
    for (var i=0; i<16; i++) {
	draw_portstate(i);
	update_gain_state(i);
    }
}


function init_portmatrix() {
    if (!parent.frames[0].permissions) {
	window.setTimeout(init_portmatrix, 100);
	return;
    }
    
    ajax_call('/wwwdata?show_labels', show_labels);
    
    init_minimatrix(16);
    
    for (var i=0; i<16; i++) {
	portstate[i] = -1;
	check_portstate(i);
    }
    
    locks = document.getElementById('locks');
    
    for (var i=0; i<16; i++) {
	var img = document.createElement('img');
	img.src = "/img/lock_23.png";
	img.id = "l_"+i;
	img.style.top = "0px";
	img.style.left = (i*23)+"px";
	img.port = i;
	img.onmousedown = lock_mousedown;
	locks.appendChild(img);
	update_lock_state(i);
	
	var img = document.createElement('img');
	img.src = "/img/portgainmode_0.png";
	img.id = "pgm_"+i;
	img.style.top = "24px";
	img.style.left = (i*23)+"px";
	img.style.zIndex = 10;
	img.port = i;
	img.onmousedown = gainmode_mousedown;
	img.style.display = 'none';
	locks.appendChild(img);

	var div = document.createElement('div');
	div.id = "g_"+i;
	div.style.position = 'absolute';
	div.style.width = "22px";
	div.style.height = "22px";
	div.style.left = (i*23+1)+"px";
	div.style.top = "24px";
	div.style.fontSize = "10px";
	div.style.textAlign = "center";
	div.style.lineHeight = "23px";    
	div.port = i;
	div.onmousedown = gain_mousedown;
	locks.appendChild(div);
    }
    
    check_gains();
    
    init_cycle_button(document.getElementById("show_labels"), ['System Labels', 'User Labels'], show_labels);
    
    window.onkeydown = cb_keydown;
    window.onkeyup = cb_keyup;

    permissions_updated();

    esc_iomask_ports(parent.frames[0].iomask_ports);
}

window.onload = init_portmatrix();
